"""Shared UI icon constants."""

ICON_BACK = "🔙"
ICON_DELETE = "🚫"
ICON_CALENDAR = "📅"
ICON_PALETTE = "🎨"
ICON_BODY = "📝"
ICON_PERSON = "🧑"
ICON_TAB_DOC = "📝"
ICON_TAB_USERS = "😃"
ICON_BOARD = "🪟"
ICON_COLUMN = "📊"
ICON_CARD = "🃏"
ICON_EDIT = "📝"
ICON_MOVE_LEFT = "👈"
ICON_MOVE_RIGHT = "👉"
ICON_MOVE_TO = "📦"
ICON_CONFIRM = "✅"
ICON_CHECKED = "✅"
ICON_UNCHECKED = "⬜"
ICON_BLOCKED = "🚧"
ICON_SETTINGS = "🔧"
ICON_SYNC_IDLE = "🟢"
ICON_SYNC_ACTIVE = "⚡️"
ICON_SYNC_PAUSED = "🔴"
ICON_SYNC_CONFLICT = "❌"
